
from typing import final
import acoustid

def signature(path):
    return acoustid.fingerprint_file(path)

import subprocess
import numpy
import os

# step size (in points) of cross correlation
step = 1

# minimum number of points that must overlap in cross correlation
# exception is raised if this cannot be met
min_overlap = 20

# report match when cross correlation has a peak exceeding threshold
threshold = 0.5

# calculate fingerprint
# Generate file.mp3.fpcalc by "fpcalc -raw -length 500 file.mp3"
def calculate_fingerprints(filename: str, length: int):
    if os.path.exists(filename + '.fpcalc'):
        print("Found precalculated fingerprint for %s" % (filename))
        f = open(filename + '.fpcalc', "r")
        fpcalc_out = ''.join(f.readlines())
        f.close()
    else:
        print("Calculating fingerprint by fpcalc for %s" % (filename))


    fpcalc_out = str()

    try:
        fpcalc_out = str(subprocess.check_output(['fpcalc', '-raw', filename]))
    except subprocess.CalledProcessError as e:
        fpcalc_out = str(e.output)

    fpcalc_out = fpcalc_out.strip().replace('\\n', '').replace("'", "")

    fingerprint_index = fpcalc_out.find('FINGERPRINT=') + 12
    # convert fingerprint to list of integers
    fingerprints = list(map(int, fpcalc_out[fingerprint_index:].split(',')))

    return fingerprints

# returns correlation between lists
def correlation(listx, listy):
    if len(listx) == 0 or len(listy) == 0:
        # Error checking in main program should prevent us from ever being
        # able to get here.
        raise Exception('Empty lists cannot be correlated.')
    if len(listx) > len(listy):
        listx = listx[:len(listy)]
    elif len(listx) < len(listy):
        listy = listy[:len(listx)]

    covariance = 0
    for i in range(len(listx)):
        covariance += 32 - bin(listx[i] ^ listy[i]).count("1")

    covariance = covariance / float(len(listx))

    correlation = covariance / 32

    if correlation is None:
        return

    return correlation


# return cross correlation, with listy offset from listx
def cross_correlation(listx, listy, offset):
    if offset > 0:
        listx = listx[offset:]
        listy = listy[:len(listx)]
    elif offset < 0:
        offset = -offset
        listy = listy[offset:]
        listx = listx[:len(listy)]
    if min(len(listx), len(listy)) < min_overlap:
        # Error checking in main program should prevent us from ever being
        # able to get here.
        return
    #raise Exception('Overlap too small: %i' % min(len(listx), len(listy)))
    corr = correlation(listx, listy)
    return corr

# cross correlate listx and listy with offsets from span
def compare(listx, listy, span, step):
    if span > min(len(listx), len(listy)):
        # Error checking in main program should prevent us from ever being
        # able to get here.
        raise Exception('span >= sample size: %i >= %i\n'
                        % (span, min(len(listx), len(listy)))
                        + 'Reduce span, reduce crop or increase sample_time.')

    corr_xy = []

    # for offset in numpy.arange(-span, 1, step):
    for offset in numpy.arange(-span, span + 1, step):
        tmp_corr = cross_correlation(listx, listy, offset)
        corr_xy.append(tmp_corr)

    corr_xy = list(filter(None, corr_xy))

    return corr_xy

# return index of maximum value in list
def max_index(listx):
    max_index = 0
    max_value = listx[0]
    for i, value in enumerate(listx):
        if value > max_value:
            max_value = value
            max_index = i
    return max_index

def get_max_corr(corr, source, target, span):
    max_corr_index = max_index(corr)
    max_corr_offset = -span + max_corr_index * step
    print("max_corr_index = ", max_corr_index, "max_corr_offset = ", max_corr_offset)
    # report matches
    if corr[max_corr_index] > threshold:
        print("File A: %s" % (source))
        print("File B: %s" % (target))
        print('Match with correlation of %.2f%% at offset %i'
             % (corr[max_corr_index] * 100.0, max_corr_offset))


def correlate(source, target, length, span):
    fingerprint_source = calculate_fingerprints(target, length)
    # print(len(fingerprint_source))

    fingerprint_target = calculate_fingerprints(source, 59)
    # print(len(fingerprint_target))

    corr = compare(fingerprint_source, fingerprint_target, span, step)

    max_corr_offset = get_max_corr(corr, source, target, span)
